/*****************************************************************************
* | File      	:		LCD_Driver.c
* | Author      :   Waveshare team
* | Function    :   Electronic paper driver
* | Info        :
*----------------
* |	This version:   V1.0
* | Date        :   2018-11-18
* | Info        :   
#
# Permission is hereby granted, free of UBYTEge, to any person obtaining a copy
# of this software and associated documnetation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to  whom the Software is
# furished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS OR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
******************************************************************************/
#include "LCD_Driver.h"

/*******************************************************************************
function:
	Hardware reset
*******************************************************************************/
static void LCD_Reset(void)
{
	DEV_Digital_Write(DEV_CS_PIN,0);
  DEV_Digital_Write(DEV_RST_PIN,1);;
	DEV_Delay_ms(200);
	DEV_Digital_Write(DEV_RST_PIN,0);;
	DEV_Delay_ms(200);
	DEV_Digital_Write(DEV_RST_PIN,1);; 
	DEV_Delay_ms(200);
}


/*******************************************************************************
function:
		Write register address and data
*******************************************************************************/
void LCD_WriteData_Byte(UBYTE da) 
{	
	DEV_Digital_Write(DEV_CS_PIN,0);
	DEV_Digital_Write(DEV_DC_PIN,1);
	DEV_SPI_WRITE(da);
	DEV_Digital_Write(DEV_CS_PIN,1);
	DEV_Digital_Write(DEV_DC_PIN,1);

}  

void LCD_WriteData_Word(UDOUBLE da)
{
	unsigned char  R,B,G;
	R= ((da>>16) & 0xff);
	G= ((da>>8) & 0xff);
	B=	da  & 0xff;
	DEV_Digital_Write(DEV_CS_PIN,0);
  DEV_Digital_Write(DEV_DC_PIN,1);
	DEV_SPI_WRITE(R);
	DEV_SPI_WRITE(G);
	DEV_SPI_WRITE(B);
	DEV_Digital_Write(DEV_CS_PIN,1);
	DEV_Digital_Write(DEV_DC_PIN,1); 
}	  

void LCD_WriteReg(UBYTE da)	 
{	
	DEV_Digital_Write(DEV_CS_PIN,0);
	DEV_Digital_Write(DEV_DC_PIN,0);
	DEV_SPI_WRITE(da);
	DEV_Digital_Write(DEV_DC_PIN,1);
}
/*******************************************************************************
function: Setting backlight
parameter	:
		value : Range 0~255   Duty cycle is value/255	
*******************************************************************************/
void LCD_SetBacklight(UWORD Value)
{
	LCD_WriteReg(0x51);
	LCD_WriteData_Byte(Value);
}

void LCD_SetREG(UWORD reg,UWORD Value)
{
	LCD_WriteReg(reg);
	LCD_WriteData_Byte(Value);
}


/******************************************************************************
function:	
		Common register initialization
******************************************************************************/
void LCD_Init(void)
{
	
	LCD_Reset();
	
	
	LCD_WriteReg(0xFE);
	LCD_WriteData_Byte(0x04);
	                 
	LCD_WriteReg(0x00);
	LCD_WriteData_Byte(0xdc);
	                 
	LCD_WriteReg(0x01);
	LCD_WriteData_Byte(0x00);
	                 
	LCD_WriteReg(0x02);
	LCD_WriteData_Byte(0x02);
	                 
	LCD_WriteReg(0x03);
	LCD_WriteData_Byte(0x00);
	                 
	LCD_WriteReg(0x04);
	LCD_WriteData_Byte(0x00);
	                 
	LCD_WriteReg(0x05);
	LCD_WriteData_Byte(0x03);
	                 
	LCD_WriteReg(0x06);
	LCD_WriteData_Byte(0x16);
	                 
	LCD_WriteReg(0x07);
	LCD_WriteData_Byte(0x13);
	                 
	LCD_WriteReg(0x08);
	LCD_WriteData_Byte(0x08);
	                 
	LCD_WriteReg(0x09);
	LCD_WriteData_Byte(0xdc);
	                 
	LCD_WriteReg(0x0a);
	LCD_WriteData_Byte(0x00);
	                 
	LCD_WriteReg(0x0b);
	LCD_WriteData_Byte(0x02);
	                 
	LCD_WriteReg(0x0c);
	LCD_WriteData_Byte(0x00);
	                 
	LCD_WriteReg(0x0d);
	LCD_WriteData_Byte(0x00);
	                 
	LCD_WriteReg(0x0e);
	LCD_WriteData_Byte(0x02);
	                 
	LCD_WriteReg(0x0f);
	LCD_WriteData_Byte(0x16);
	                 
	LCD_WriteReg(0x10);
	LCD_WriteData_Byte(0x18);
	                 
	LCD_WriteReg(0x11);
	LCD_WriteData_Byte(0x08);
	                 
	LCD_WriteReg(0x12);
	LCD_WriteData_Byte(0xC2);
	                 
	LCD_WriteReg(0x13);
	LCD_WriteData_Byte(0x00);
	                 
	LCD_WriteReg(0x14);
	LCD_WriteData_Byte(0x34);
	                 
	LCD_WriteReg(0x15);
	LCD_WriteData_Byte(0x05);
	                 
	LCD_WriteReg(0x16);
	LCD_WriteData_Byte(0x00);
	                 
	LCD_WriteReg(0x17);
	LCD_WriteData_Byte(0x03);
	
	LCD_WriteReg(0x18);
	LCD_WriteData_Byte(0x15);
	
	LCD_WriteReg(0x19);
	LCD_WriteData_Byte(0x41);
	
	LCD_WriteReg(0x1a);
	LCD_WriteData_Byte(0x00);
	LCD_WriteReg(0x1b);
	LCD_WriteData_Byte(0xdc);
	LCD_WriteReg(0x1c);
	LCD_WriteData_Byte(0x00);
	LCD_WriteReg(0x1d);
	LCD_WriteData_Byte(0x04);
	LCD_WriteReg(0x1e);
	LCD_WriteData_Byte(0x00);
	LCD_WriteReg(0x1f);
	LCD_WriteData_Byte(0x00);
	LCD_WriteReg(0x20);
	LCD_WriteData_Byte(0x03);
	LCD_WriteReg(0x21);
	LCD_WriteData_Byte(0x16);
	LCD_WriteReg(0x22);
	LCD_WriteData_Byte(0x18);
	LCD_WriteReg(0x23);
	LCD_WriteData_Byte(0x08);
	LCD_WriteReg(0x24);
	LCD_WriteData_Byte(0xdc);
	LCD_WriteReg(0x25);
	LCD_WriteData_Byte(0x00);
	LCD_WriteReg(0x26);
	LCD_WriteData_Byte(0x04);
	LCD_WriteReg(0x27);
	LCD_WriteData_Byte(0x00);
	LCD_WriteReg(0x28);
	LCD_WriteData_Byte(0x00);
	LCD_WriteReg(0x29);
	LCD_WriteData_Byte(0x01);
	LCD_WriteReg(0x2a);
	LCD_WriteData_Byte(0x16);
	LCD_WriteReg(0x2b);
	LCD_WriteData_Byte(0x18);
	LCD_WriteReg(0x2d);
	LCD_WriteData_Byte(0x08);
	LCD_WriteReg(0x4c);
	LCD_WriteData_Byte(0x99);
	LCD_WriteReg(0x4d);
	LCD_WriteData_Byte(0x00);
	LCD_WriteReg(0x4e);
	LCD_WriteData_Byte(0x00);
	LCD_WriteReg(0x4f);
	LCD_WriteData_Byte(0x00);
	LCD_WriteReg(0x50);
	LCD_WriteData_Byte(0x01);
	LCD_WriteReg(0x51);
	LCD_WriteData_Byte(0x0A);
	LCD_WriteReg(0x52);
	LCD_WriteData_Byte(0x00);
	LCD_WriteReg(0x5a);
	LCD_WriteData_Byte(0xe4);
	LCD_WriteReg(0x5e);
	LCD_WriteData_Byte(0x77);
	LCD_WriteReg(0x5f);
	LCD_WriteData_Byte(0x77);
	LCD_WriteReg(0x60);
	LCD_WriteData_Byte(0x34);
	LCD_WriteReg(0x61);
	LCD_WriteData_Byte(0x02);
	LCD_WriteReg(0x62);
	LCD_WriteData_Byte(0x81);
                   
	LCD_WriteReg(0xFE);
	LCD_WriteData_Byte(0x07);
	LCD_WriteReg(0x07);
	LCD_WriteData_Byte(0x4F);
                   
	LCD_WriteReg(0xFE);
	LCD_WriteData_Byte(0x01);
	LCD_WriteReg(0x04);
	LCD_WriteData_Byte(0x80);
	LCD_WriteReg(0x05);
	LCD_WriteData_Byte(0x65);
	LCD_WriteReg(0x06);
	LCD_WriteData_Byte(0x1E);
	LCD_WriteReg(0x0E);
	LCD_WriteData_Byte(0x8B);
	LCD_WriteReg(0x0F);
	LCD_WriteData_Byte(0x8B);
	LCD_WriteReg(0x10);
	LCD_WriteData_Byte(0x11);
	LCD_WriteReg(0x11);
	LCD_WriteData_Byte(0xA2);
	LCD_WriteReg(0x12);
	LCD_WriteData_Byte(0x80);
	LCD_WriteReg(0x14);
	LCD_WriteData_Byte(0x81);
	LCD_WriteReg(0x15);
	LCD_WriteData_Byte(0x82);
	LCD_WriteReg(0x18);
	LCD_WriteData_Byte(0x45);
	LCD_WriteReg(0x19);
	LCD_WriteData_Byte(0x34);
	LCD_WriteReg(0x1A);
	LCD_WriteData_Byte(0x10);
	LCD_WriteReg(0x1C);
	LCD_WriteData_Byte(0x57);
	LCD_WriteReg(0x1D);
	LCD_WriteData_Byte(0x02);
	LCD_WriteReg(0x21);
	LCD_WriteData_Byte(0xF8);
	LCD_WriteReg(0x22);
	LCD_WriteData_Byte(0x90);
	LCD_WriteReg(0x23);
	LCD_WriteData_Byte(0x00);
	LCD_WriteReg(0x25);
	LCD_WriteData_Byte(0x0A);
	LCD_WriteReg(0x26);
	LCD_WriteData_Byte(0x46);
	LCD_WriteReg(0x2A);
	LCD_WriteData_Byte(0x47);
	LCD_WriteReg(0x2B);
	LCD_WriteData_Byte(0xFF);
	LCD_WriteReg(0x2D);
	LCD_WriteData_Byte(0xFF);
	LCD_WriteReg(0x2F);
	LCD_WriteData_Byte(0xAE);
	LCD_WriteReg(0x37);
	LCD_WriteData_Byte(0x0C);
	LCD_WriteReg(0x3a);
	LCD_WriteData_Byte(0x00);
	LCD_WriteReg(0x3b);
	LCD_WriteData_Byte(0x20);
	LCD_WriteReg(0x3d);
	LCD_WriteData_Byte(0x0B);
	LCD_WriteReg(0x3f);
	LCD_WriteData_Byte(0x38);
	LCD_WriteReg(0x40);
	LCD_WriteData_Byte(0x0B);
	LCD_WriteReg(0x41);
	LCD_WriteData_Byte(0x0B);
	LCD_WriteReg(0x42);
	LCD_WriteData_Byte(0x11);
	LCD_WriteReg(0x43);
	LCD_WriteData_Byte(0x44);
	LCD_WriteReg(0x44);
	LCD_WriteData_Byte(0x22);
	LCD_WriteReg(0x45);
	LCD_WriteData_Byte(0x55);
	LCD_WriteReg(0x46);
	LCD_WriteData_Byte(0x33);
	LCD_WriteReg(0x47);
	LCD_WriteData_Byte(0x66);
	LCD_WriteReg(0x4c);
	LCD_WriteData_Byte(0x11);
	LCD_WriteReg(0x4d);
	LCD_WriteData_Byte(0x44);
	LCD_WriteReg(0x4e);
	LCD_WriteData_Byte(0x22);
	LCD_WriteReg(0x4f);
	LCD_WriteData_Byte(0x55);
	LCD_WriteReg(0x50);
	LCD_WriteData_Byte(0x33);
	LCD_WriteReg(0x51);
	LCD_WriteData_Byte(0x66);
	LCD_WriteReg(0x56);
	LCD_WriteData_Byte(0x11);
	LCD_WriteReg(0x58);
	LCD_WriteData_Byte(0x44);
	LCD_WriteReg(0x59);
	LCD_WriteData_Byte(0x22);
	LCD_WriteReg(0x5a);
	LCD_WriteData_Byte(0x55);
	LCD_WriteReg(0x5b);
	LCD_WriteData_Byte(0x33);
	LCD_WriteReg(0x5c);
	LCD_WriteData_Byte(0x66);
	LCD_WriteReg(0x61);
	LCD_WriteData_Byte(0x11);
	LCD_WriteReg(0x62);
	LCD_WriteData_Byte(0x44);
	LCD_WriteReg(0x63);
	LCD_WriteData_Byte(0x22);
	LCD_WriteReg(0x64);
	LCD_WriteData_Byte(0x55);
	LCD_WriteReg(0x65);
	LCD_WriteData_Byte(0x33);
	LCD_WriteReg(0x66);
	LCD_WriteData_Byte(0x66);
	LCD_WriteReg(0x6D);
	LCD_WriteData_Byte(0x90);
	LCD_WriteReg(0x6E);
	LCD_WriteData_Byte(0x40);
	LCD_WriteReg(0x70);
	LCD_WriteData_Byte(0xA5);
	LCD_WriteReg(0x72);
	LCD_WriteData_Byte(0x04);
	LCD_WriteReg(0x73);
	LCD_WriteData_Byte(0x15);
                   
	LCD_WriteReg(0xFE);
	LCD_WriteData_Byte(0x0A);
	LCD_WriteReg(0x29);
	LCD_WriteData_Byte(0x90);
                   
	LCD_WriteReg(0xFE);
	LCD_WriteData_Byte(0x05);
	LCD_WriteReg(0x05);
	LCD_WriteData_Byte(0x1F);
                   
	LCD_WriteReg(0xFE);
	LCD_WriteData_Byte(0x00);
	LCD_WriteReg(0x35);
	LCD_WriteData_Byte(0x00);
	LCD_WriteReg(0x3A);
	LCD_WriteData_Byte(0x77);

	LCD_WriteReg(0x53);
	LCD_WriteData_Byte(0x20);

	LCD_WriteReg(0x13);
	LCD_WriteReg(0x20);
	LCD_WriteReg(0x36);//
	LCD_WriteData_Byte(0x30);//
                   
	LCD_WriteReg(0x11);    //Exit Sleep 
	DEV_Delay_ms(200);      
	LCD_WriteReg(0x29);    //Display on 
	
} 

/******************************************************************************
function:	Set the cursor position
parameter	:
		Xstart: 	Start UWORD x coordinate
	  Ystart:		Start UWORD y coordinate
	  Xend  :		End UWORD coordinates
	  Yend  :		End UWORD coordinatesen
******************************************************************************/
void LCD_SetCursor(UWORD Xstart, UWORD Ystart, UWORD Xend, UWORD  Yend)
{ 
	LCD_WriteReg(0x2a);
	LCD_WriteData_Byte(Xstart>>8);
	LCD_WriteData_Byte(Xstart);
	LCD_WriteData_Byte(Xend >>8);
	LCD_WriteData_Byte(Xend);

	LCD_WriteReg(0x2b);
	LCD_WriteData_Byte(Ystart>>8);
	LCD_WriteData_Byte(Ystart);
	LCD_WriteData_Byte(Yend>>8);
	LCD_WriteData_Byte(Yend);

	LCD_WriteReg(0x2C);
}

/******************************************************************************
function:	Clear screen function, refresh the screen to a certain color
parameter	:
		Color :		The color you want to clear all the screen
******************************************************************************/
void LCD_Clear(UDOUBLE Color)
{
#if 0
	UWORD i,j;  	
	LCD_SetCursor(0,0,LCD_WIDTH-1,LCD_HEIGHT-1);
	for(i = 0; i < LCD_WIDTH; i++){
		for(j = 0; j < LCD_HEIGHT; j++){
			LCD_WriteData_Word(Color);
		}
	 }
#else 
	UBYTE R, B, G;
	R = ((Color>>16) & 0xff);
	G = ((Color>>8) & 0xff);
	B =	Color & 0xff;

	LCD_SetCursor(0, 0, LCD_WIDTH - 1, LCD_HEIGHT - 1);

	UWORD i,j;  	
	DEV_Digital_Write(DEV_DC_PIN, 1);
	DEV_Digital_Write(DEV_CS_PIN, 0);
	for(i = 0; i < LCD_WIDTH; i++){
		for(j = 0; j < LCD_HEIGHT; j++){
			DEV_SPI_WRITE(R);
			DEV_SPI_WRITE(G);
			DEV_SPI_WRITE(B);
		}
	}
	DEV_Digital_Write(DEV_CS_PIN, 1);
	DEV_Digital_Write(DEV_DC_PIN, 1);
#endif
}

/******************************************************************************
function:	Refresh a certain area to the same color
parameter	:
		Xstart: 	Start UWORD x coordinate
	  Ystart:		Start UWORD y coordinate
	  Xend  :		End UWORD coordinates
	  Yend  :		End UWORD coordinates
	  color :		Set the color
******************************************************************************/
void LCD_ClearWindow(UWORD Xstart, UWORD Ystart, UWORD Xend, UWORD Yend,UWORD color)
{          
	UWORD i,j; 
	LCD_SetCursor(Xstart, Ystart, Xend-1,Yend-1);
	for(i = Ystart; i <= Yend-1; i++){													   	 	
		for(j = Xstart; j <= Xend-1; j++){
			LCD_WriteData_Word(color);
		}
	} 					  	    
}  

/******************************************************************************
function: Set the color of an area
parameter	:
		Xstart: 	Start UWORD x coordinate
	  Ystart:		Start UWORD y coordinate
	  Xend  :		End UWORD coordinates
	  Yend  :		End UWORD coordinates
	  Color :		Set the color
******************************************************************************/
void LCD_SetWindowColor(UWORD Xstart, UWORD Ystart, UWORD Xend, UWORD Yend,UWORD  Color)
{
	LCD_SetCursor( Xstart,Ystart,Xend,Yend);
	LCD_WriteData_Word(Color); 	    
}

/******************************************************************************
function: Draw a Point
parameter	:
		X			: 	Set the X coordinate
	  Y			:		Set the Y coordinate
	  Color :		Set the color
******************************************************************************/
void LCD_Draw_Point(UWORD x, UWORD y, UDOUBLE Color)
{
	LCD_SetCursor(x,y,x,y);
	LCD_WriteData_Word(Color); 	    
} 



